package com.yingji.quartz;

import cn.hutool.core.collection.CollectionUtil;
import cn.hutool.core.util.StrUtil;
import cn.hutool.http.HttpRequest;
import cn.hutool.json.JSONUtil;
import com.yingji.entity.Fire;
import com.yingji.entity.QuartzLog;
import com.yingji.service.FireService;
import com.yingji.service.QuartzLogService;
import com.yingji.utils.SmsUtil;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.context.annotation.Configuration;
import org.springframework.scheduling.annotation.Async;
import org.springframework.scheduling.annotation.Scheduled;

import javax.annotation.Resource;
import java.time.Duration;
import java.time.LocalDateTime;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * 119数据保存定时任务
 *
 * @author wu
 * @since 2024/05/06 10:18
 */
@Configuration
public class FireQuartz {

    public static final Logger log = LoggerFactory.getLogger(FireQuartz.class);

    @Resource
    private FireService fireService;


    @Resource
    private QuartzLogService quartzLogService;

    @Resource
    private SmsUtil smsUtil;

    /**
     * 每天早上9点判断数据
     */
    @Async
    @Scheduled(cron = "0 0 9 * * ? ")
    public void sendMes() {
        // 获取数据库中最新的创建时间
        LocalDateTime newTime = fireService.findNewTime();
        // 获取当前时间
        LocalDateTime currentTime = LocalDateTime.now();
        // 计算两者之间的持续时间
        Duration duration = Duration.between(newTime, currentTime);
        // 判断时间差是否大于12小时
        if (duration.toHours() > 12) {
            // 调用短信接口
            smsUtil.sendSms("119原始接口超过12小时无数据", null,null);
        }
    }

    /**
     * 保存所有数据
     */
    @Async
    @Scheduled(cron = "0 */5 * * * ? ")
    public void savaData() {
        log.info("119接口开始" + LocalDateTime.now());
        QuartzLog quartzLog = new QuartzLog();
        quartzLog.setInterfaceName(2);
        quartzLog.setCreateTime(LocalDateTime.now());
        try {
            // 获取token
            String token = fireService.getToken();
            if (StrUtil.isNotEmpty(token)) {
                // 根据id查询数据保存
                List<Fire> list = fireService.findDataList(token);
                // 110算法接口
                if (CollectionUtil.isNotEmpty(list)) {
                    fireService.saveAll(list);
                    try {
                        for (Fire fire : list) {
                            link119Algorithm(fire.getId());
                        }
                    } catch (Exception e) {
                        // 调用短信接口
                        smsUtil.sendSms("119算法事件接口异常", null,"119sfError");
                    }
                }
                quartzLog.setStatus(1);
                log.info("119接口结束" + LocalDateTime.now());
            } else {
                quartzLog.setStatus(2);
                // 调用短信接口
                smsUtil.sendSms("119原始接口获取token失败", null,null);
                log.error("============119算法接口获取token失败119接口结束=============");
            }
        } catch (Exception e) {
            quartzLog.setStatus(2);
            // 调用短信接口
//            smsUtil.sendSms("119原始接口调用失败", null,"119ErrorMes");
            smsUtil.sendSms("119原始接口调用失败", null,null);
        } finally {
            quartzLogService.save(quartzLog);
        }
    }

    /**
     * 120算法接口
     */
    public void link119Algorithm(String id) {
        log.info("============119和110对比算法接口开始" + LocalDateTime.now() + "=============");
        Map<String, String> map = new HashMap<>();
        map.put("id", id);
        String body = JSONUtil.toJsonStr(map);
        String response = HttpRequest.post("http://localhost:9002/link119_algorithm").body(body).execute().body();
        log.info("++++++119和110对比算法响应数据++++++" + response);
        log.info("============119和110对比算法接口结束" + LocalDateTime.now() + "=============");
    }
}
