package com.yingji.service.impl;

import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.collection.CollectionUtil;
import cn.hutool.core.util.StrUtil;
import cn.hutool.http.HttpException;
import cn.hutool.http.HttpRequest;
import cn.hutool.json.JSONUtil;
import com.alibaba.fastjson2.JSONObject;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.yingji.entity.AcceptEvents;
import com.yingji.entity.Amb;
import com.yingji.entity.Event;
import com.yingji.entity.EventClass;
import com.yingji.entity.Task;
import com.yingji.entity.dto.response.EventsResponse;
import com.yingji.entity.dto.response.TaskInfos;
import com.yingji.redis.RedisCache;
import com.yingji.service.AcceptService;
import com.yingji.service.AccidentService;
import com.yingji.service.AmbService;
import com.yingji.service.EventClassService;
import com.yingji.service.EventService;
import com.yingji.service.TaskService;
import com.yingji.utils.SmsUtil;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.time.LocalDate;
import java.time.LocalDateTime;
import java.time.format.DateTimeFormatter;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.concurrent.TimeUnit;

/**
 * @author wu
 * @since 2024/5/10 上午10:39
 */
@Service("accidentService")
public class AccidentServiceImpl implements AccidentService {

    public static final Logger log = LoggerFactory.getLogger(AccidentServiceImpl.class);

    @Value("${getAuthorizeValidateUrl}")
    private String getAuthorizeValidateUrl;

    @Value("${getAuthorizeTokenUrl}")
    private String getAuthorizeTokenUrl;

    @Value("${getAccidentUrl}")
    private String getAccidentUrl;

    @Resource
    private RedisCache redisCache;

    @Resource
    private EventService eventService;

    @Resource
    private TaskService taskService;

    @Resource
    private AmbService ambService;

    @Resource
    private AcceptService acceptService;

    @Resource
    private EventClassService eventClassService;

    @Resource
    private SmsUtil smsUtil;
//    public static void main(String[] args) {
//
//        JSONObject json = JSONObject.parse(s);
//        Object data = json.get("data");
//        String recordsStr = JSONUtil.toJsonStr(data);
//        List<EventsResponse> list = JSONUtil.toList(recordsStr, EventsResponse.class);
//        System.out.println(list);
//    }

    /**
     * 获取token校验
     */
    @Override
    public String getAuthorizeValidate() {
        String validate = redisCache.getCacheObject("validate");
        if (StrUtil.isEmpty(validate)) {
            // 获取当前的时间
            LocalDateTime localDateTime = LocalDateTime.now();
            DateTimeFormatter formatter = DateTimeFormatter.ofPattern("yyyy-MM-dd HH:mm");
            String accessDateTime = localDateTime.format(formatter);
            Map<String, Object> map = new HashMap<>();
            map.put("AKey", "admin");
            map.put("ASecret", "anchor120.");
            map.put("AccessDateTime", accessDateTime);
            String parse = JSONUtil.toJsonStr(map);
            String body = HttpRequest.post(getAuthorizeValidateUrl).body(parse).execute().body();
            JSONObject json = JSONObject.parse(body);
            validate = (String) json.get("data");
            if (StrUtil.isNotEmpty(validate)) {
                redisCache.setCacheObject("validate", validate, 40, TimeUnit.MINUTES);
            }
        }
        log.info("=======================================");
        log.info("120token校验为:" + validate);
        return validate;
    }

    /**
     * 获取token
     */
    @Override
    public String getAuthorizeToken() {
        String accessToken;
        accessToken = redisCache.getCacheObject("accessToken");
        if (StrUtil.isEmpty(accessToken)) {
            // 获取授权码
            String validate = getAuthorizeValidate();
            Map<String, Object> map = new HashMap<>();
            map.put("Data", validate);
            String parse = JSONUtil.toJsonStr(map);
            String body = HttpRequest.post(getAuthorizeTokenUrl).body(parse).execute().body();
            JSONObject json = JSONObject.parse(body);
            Object data = json.get("data");
            if (BeanUtil.isNotEmpty(data)) {
                String recordsStr = JSONUtil.toJsonStr(data);
                JSONObject res = JSONObject.parse(recordsStr);
                accessToken = (String) res.get("token");
                if (StrUtil.isNotEmpty(accessToken)) {
                    redisCache.setCacheObject("accessToken", accessToken, 40, TimeUnit.MINUTES);
                }
            }
            log.info("=======================================");
            log.info("120token为:" + accessToken);
        }
        return accessToken;
    }

    /**
     * 保存数据
     *
     * @param accessToken token
     */
    @Override
    public void getAccidentUrl(String accessToken) {
        // 获取今天的日期
        LocalDate today = LocalDate.now();
        // 获取明天的日期
        LocalDate tomorrow = today.plusDays(1);
        // 定义日期格式化对象
        DateTimeFormatter formatter = DateTimeFormatter.ofPattern("yyyy-MM-dd");
        // 格式化日期为字符串
        String todayStr = today.format(formatter) + " 00:00:00";
        String tomorrowStr = tomorrow.format(formatter) + " 00:00:00";
        Map<String, Object> map = new HashMap<>();
        map.put("StartDateTime", todayStr);
        map.put("EndDateTime", tomorrowStr);
        String parse = JSONUtil.toJsonStr(map);
        String body = HttpRequest.post(getAccidentUrl).header("Authorization", "Bearer " + accessToken).body(parse).execute().body();
        log.error("120接收到的数据为=======================" + body);
        if (body.contains("Time-out")) {
            return ;
        }
        JSONObject json = JSONObject.parse(body);
        Object data = json.get("data");
        if (BeanUtil.isNotEmpty(data)) {
            saveResponseData(data);
        } else {
            redisCache.deleteObject("validate");
            redisCache.deleteObject("accessToken");
        }
    }

    /**
     * 保存响应数据
     *
     * @param data 响应数据
     */
    private void saveResponseData(Object data) {
        List<EventsResponse> list;
        String recordsStr = JSONUtil.toJsonStr(data);
        if (StrUtil.isNotEmpty(recordsStr)) {
            // json数据转换成实体类
            list = JSONUtil.toList(recordsStr, EventsResponse.class);
            if (CollectionUtil.isNotEmpty(list)) {
                // 获取事故数据保存
                List<Event> events = BeanUtil.copyToList(list, Event.class);
                for (Event event : events) {
                    // 新增或编辑事故(Event) 表
                    QueryWrapper<Event> wrapper = new QueryWrapper<>();
                    wrapper.eq("event_id", event.getEventId());
                    Event one = eventService.getOne(wrapper);
                    if (BeanUtil.isEmpty(one)) {
                        event.setCreateTime(LocalDateTime.now());
                        event.setUpdateTime(LocalDateTime.now());
                        if (StrUtil.isNotEmpty(event.getAccidentType())) {
                            EventClass eventClass = eventClassService.findByAccidentType(event.getAccidentType());
                            if (BeanUtil.isNotEmpty(eventClass)) {
                                event.setAiClass(eventClass.getAiClass());
                            }
                        }
                        eventService.save(event);
                        // 调用120和110事件对比接口
                        linkAlgorithm(event.getEventId());
                        log.info("======调用120和110事件对比接口======");
                    } else {
                        event.setUpdateTime(LocalDateTime.now());
                        eventService.updateById(event);
                    }
                }
                for (EventsResponse res : list) {
                    List<AcceptEvents> acceptEvents = res.getAcceptEvents();
                    // 保存或编辑受理调度信息
                    for (AcceptEvents acceptEvent : acceptEvents) {
                        QueryWrapper<AcceptEvents> wrapper = new QueryWrapper<>();
                        wrapper.eq("accept_id", acceptEvent.getAcceptId());
                        AcceptEvents one = acceptService.getOne(wrapper);
                        if (BeanUtil.isEmpty(one)) {
                            acceptService.save(acceptEvent);
                        } else {
                            acceptService.updateById(acceptEvent);
                        }
                    }
                    List<TaskInfos> taskInfos = res.getTaskInfos();
                    for (TaskInfos taskInfo : taskInfos) {
                        // 保存或编辑任务信息(Task) 表实体类
                        Task task = taskInfo.getTask();
                        if (BeanUtil.isNotEmpty(task)) {
                            QueryWrapper<Task> wrapper = new QueryWrapper<>();
                            wrapper.eq("task_id", task.getTaskId());
                            Task one = taskService.getOne(wrapper);
                            if (BeanUtil.isEmpty(one)) {
                                taskService.save(task);
                                Amb amb = taskInfo.getAmb();
                                if (BeanUtil.isNotEmpty(amb)) {
                                    amb.setTaskId(task.getTaskId());
                                    ambService.save(amb);
                                }
                            } else {
                                taskService.updateById(task);
                            }
                        }
                    }
                }
            }
        }
    }

    /**
     * 120算法接口
     */
    public void linkAlgorithm(String eventId) {
        try {
            log.info("============120和110对比算法接口开始" + LocalDateTime.now() + "=============");
            Map<String, String> map = new HashMap<>();
            map.put("event_id", eventId);
            String body = JSONUtil.toJsonStr(map);
            String response = HttpRequest.post("http://localhost:9002/link_algorithm").body(body).execute().body();
            log.info("++++++120和110对比算法响应数据++++++" + response);
            log.info("============120和110对比算法接口结束" + LocalDateTime.now() + "=============");
        } catch (HttpException e) {
            // 调用短信接口
            smsUtil.sendSms("120算法事件接口异常", null, "120sfError");
        }
    }

    /**
     * 获取数据库最新创建时间
     *
     * @return 数据库最新创建时间
     */
    @Override
    public LocalDateTime findNewTime() {
        return eventService.findNewTime();
    }
}